library HmsVlcAddon;

{$RTTI EXPLICIT METHODS([]) PROPERTIES([]) FIELDS([])}
{$WEAKLINKRTTI ON}

uses
  Types,
  Messages,
  SysUtils,
  Classes,
  ActiveX,
  Windows,
  Controls,
  Forms,
  Menus,
  Variants,
  HmsAddonsIntfTypes in 'HmsAddonsIntfTypes.pas',
  AXVLC_TLB in 'AXVLC_TLB.pas';

{$R *.res}

type
  THmsAddonList = class(TComponent, IHmsAddonList, IHmsAddonTools)
  private
    FNeedToUninitialize: Boolean;
    FObjectList: TList;
    procedure FreeObjects;
  protected
    function AddObject(aObject: TComponent): TComponent;

    property ObjectList: TList read FObjectList;
  public
    constructor Create; reintroduce;
    destructor Destroy; override;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  { IHmsAddonTools }
    function Setup(aOwner: THandle; var aReload: LongBool): HResult; safecall;
    function Update(var aFlags: Integer; var aResult: OleVariant): HResult; safecall;
  { IHmsAddonList }
    function GetCount(var aCount: Integer): HResult; safecall;
    function GetAddonInfo(aIndex: Integer; var aClassID, aInterfaceID: TGUID;
       var aTitle, aDescription, aRequiredVersion, aCheckedOnVersion: OleVariant): HResult; safecall;
    function GetClassObject(const aClassID: TGUID; const aInterfaceID: TGUID; out aObject: OleVariant): HResult; safecall;
    function CanUnloadNow: HResult; safecall;
  end;

  THmsMainFormControl = class(TComponent, IHmsMainFormControl)
  private
    FControl: TWinControl;
    FPlayer: TVLCPlugin2;
    FPopupMenu: TPopupMenu;
    procedure CreatePopupMenu;
    procedure DoSelectAudio(Sender: TObject);
    procedure DoSelectChapter(Sender: TObject);
    procedure DoSelectItem(Sender: TObject);
    procedure DoSelectSubtitle(Sender: TObject);
    procedure DoSelectTitle(Sender: TObject);
    procedure DoSelectVideo(Sender: TObject);
    procedure DoSetupMenu(Sender: TObject);
  protected
    function CreateControl(aParent: THandle; const aReserved: IUnknown; var aControl: THandle): HResult; safecall;
    function DestroyControl(aControl: THandle): HResult; safecall;
    function GetCaption(var aCaption: OleVariant): HResult; safecall;

    function ExecuteAction(const aActionName: OleVariant; const aActionParameters: OleVariant): HResult; reintroduce; safecall;

    function GetActionCount(var aCount: Integer): HResult; safecall;
    function GetActionInfo(aIndex: Integer; var aActionName, aActionTitle, aActionDescription: OleVariant): HResult; safecall;

    function SetFocus: HResult; safecall;
  public
    destructor Destroy; override;
  end;

{ THmsAddonList }

type
  THmsAddonInfo = record
    ClassID: TGUID;
    InterfaceID: TGUID;
    Title: string;
    Description: string;
    RequiredVersion: string;
    CheckedOnVersion: string;
  end;

const
  AddonInfoCount = 1;

  CLASS_VlcMediaPlayer = '{2544FBDD-B5BC-4747-AA89-0797A6C20006}';

  csAddonTitle = 'VLC Media Player';

  AddonInfoList: array[0..AddonInfoCount - 1] of THmsAddonInfo =
    ((ClassID : CLASS_VlcMediaPlayer;
      InterfaceID: IID_HmsMainFormControl;
      Title : csAddonTitle;
      Description: ' "VLC Media Player"';
      RequiredVersion : '3.03';
      CheckedOnVersion: '3.03'));

function RegistryKeyExists(aRootKey: THandle; const aPath: string): LongBool;
var
  Key: HKey;
begin
  Result := RegOpenKeyEx(aRootKey, PChar(aPath), 0,
                          KEY_READ, Key) = ERROR_SUCCESS;
  if Result then RegCloseKey(Key)
end;

{ THmsAddonList }

constructor THmsAddonList.Create;
begin
  FNeedToUninitialize := Succeeded(CoInitialize(nil));
  inherited Create(nil);
  FObjectList := TList.Create;
end;

destructor THmsAddonList.Destroy;
begin
  FreeObjects;
  inherited Destroy;
end;

function THmsAddonList.AddObject(aObject: TComponent): TComponent;
begin
  FObjectList.Add(aObject);
  aObject.FreeNotification(Self);
  Result := aObject
end;

function THmsAddonList.GetCount(var aCount: Integer): HResult; safecall;
begin
  aCount := AddonInfoCount;
  Result := S_OK
end;

function THmsAddonList.GetAddonInfo(aIndex: Integer; var aClassID, aInterfaceID: TGUID;
       var aTitle, aDescription, aRequiredVersion, aCheckedOnVersion: OleVariant): HResult; safecall;
begin
  if (aIndex >= 0) and (aIndex < AddonInfoCount) then begin
    with AddonInfoList[aIndex] do begin
      aClassID := ClassID;
      aInterfaceID := InterfaceID;
      aTitle := Title;
      aDescription := Description;
      aRequiredVersion := RequiredVersion;
      aCheckedOnVersion := CheckedOnVersion
    end;
    Result := S_OK
  end else
    Result := E_UNEXPECTED
end;

function THmsAddonList.GetClassObject(const aClassID: TGUID; const aInterfaceID: TGUID; out aObject: OleVariant): HResult; safecall;
begin
  Result := E_NOINTERFACE;
  if IsEqualGUID(aInterfaceID, IHmsMainFormControl) then begin
    aObject := AddObject(THmsMainFormControl.Create(nil)) as IHmsMainFormControl;
    Result := S_OK
  end
end;

procedure THmsAddonList.FreeObjects;
var
  i: Integer;
begin
  for i := FObjectList.Count - 1 downto 0 do try
    TObject(FObjectList[i]).Free;
  except end;
  FObjectList.Clear
end;

function THmsAddonList.CanUnloadNow: HResult; safecall;
begin
  FreeObjects;
  if FNeedToUninitialize then
    CoUninitialize;
  Result := S_OK
end;

procedure THmsAddonList.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    FObjectList.Remove(AComponent)
end;

function THmsAddonList.Setup(aOwner: THandle; var aReload: LongBool): HResult; safecall;
begin
  aReload := MessageBox(aOwner, '  ?', '  ', MB_ICONQUESTION + MB_YESNO) = IDYES;
  Result := S_OK
end;

function THmsAddonList.Update(var aFlags: Integer; var aResult: OleVariant): HResult; safecall;
begin
  aFlags := ufIsInfoMessage;
  aResult := '  ';
  Result := S_OK
end;

{ THmsMainFormControl }

destructor THmsMainFormControl.Destroy;
begin
  FreeAndNil(FControl);
  inherited Destroy;
end;

function NewMenuItem(const ACaption: string; aTag: UIntPtr = 0; AOnClick: TNotifyEvent = nil; AChecked: Boolean = False): TMenuItem;
begin
  Result := NewItem(ACaption, 0, AChecked, True, AOnClick, 0, '');
  Result.Tag := aTag
end;

function MakeDescription(const aDescription: string; const aTemplate: string; aIndex: Integer): string;
begin
  if aDescription <> '' then
    Result := aDescription
  else
    Result := Format(aTemplate, [aIndex])
end;

procedure THmsMainFormControl.DoSetupMenu(Sender: TObject);
var
  i, iChapterCount, iTitle: Integer;
begin
  for i := 0 to FPopupMenu.Items.Count - 1 do
    FPopupMenu.Items[i].Clear;
  for i := 0 to FPlayer.video.count - 1 do
    FPopupMenu.Items[0].Add(NewMenuItem(FPlayer.video.description(i), i, DoSelectVideo, FPlayer.video.track = i));
  for i := 0 to FPlayer.audio.count - 1 do
    FPopupMenu.Items[1].Add(NewMenuItem(FPlayer.audio.description(i), i, DoSelectAudio, FPlayer.audio.track = i));
  for i := 0 to FPlayer.subtitle.count - 1 do
    FPopupMenu.Items[2].Add(NewMenuItem(FPlayer.subtitle.description(i), i, DoSelectSubtitle, FPlayer.subtitle.track = i));
  iTitle := FPlayer.input.title.track;
  for i := 0 to FPlayer.input.title.count - 1 do
    FPopupMenu.Items[3].Add(NewMenuItem(MakeDescription(FPlayer.input.title.description(i), ' %d', i), i, DoSelectTitle, iTitle = i));
  iChapterCount := FPlayer.input.chapter.countForTitle(iTitle);
  for i := 0 to iChapterCount - 1 do
    FPopupMenu.Items[4].Add(NewMenuItem(MakeDescription(FPlayer.input.chapter.description(iTitle, i), ' %d', i + 1), i, DoSelectChapter, FPlayer.input.chapter.track = i));
  if FPlayer.playlist.items.count > 1 then
    for i := 0 to FPlayer.playlist.items.count - 1 do
      FPopupMenu.Items[5].Add(NewMenuItem(MakeDescription('', ' %d', i + 1), i, DoSelectItem, FPlayer.playlist.currentItem = i));
  for i := 0 to FPopupMenu.Items.Count - 1 do
    FPopupMenu.Items[i].Visible := FPopupMenu.Items[i].Count > 0
end;

procedure THmsMainFormControl.DoSelectAudio(Sender: TObject);
begin
  if Sender is TComponent then
    FPlayer.audio.track := TComponent(Sender).Tag
end;

procedure THmsMainFormControl.DoSelectChapter(Sender: TObject);
begin
  if Sender is TComponent then
    FPlayer.input.chapter.track := TComponent(Sender).Tag
end;

procedure THmsMainFormControl.DoSelectItem(Sender: TObject);
begin
  if Sender is TComponent then
    FPlayer.playlist.playItem(TComponent(Sender).Tag)
end;

procedure THmsMainFormControl.DoSelectSubtitle(Sender: TObject);
begin
  if Sender is TComponent then
    FPlayer.subtitle.track := TComponent(Sender).Tag
end;

procedure THmsMainFormControl.DoSelectTitle(Sender: TObject);
begin
  if Sender is TComponent then
    FPlayer.input.title.track := TComponent(Sender).Tag
end;

procedure THmsMainFormControl.DoSelectVideo(Sender: TObject);
begin
  if Sender is TComponent then
    FPlayer.video.track := TComponent(Sender).Tag
end;

procedure THmsMainFormControl.CreatePopupMenu;
begin
  FPopupMenu := TPopupMenu.Create(FControl);
  FPopupMenu.OnPopup := DoSetupMenu;
  FPopupMenu.Items.Add(NewMenuItem(''));
  FPopupMenu.Items.Add(NewMenuItem(''));
  FPopupMenu.Items.Add(NewMenuItem(''));
  FPopupMenu.Items.Add(NewMenuItem(''));
  FPopupMenu.Items.Add(NewMenuItem(''));
  FPopupMenu.Items.Add(NewMenuItem(''));
  FPlayer.PopupMenu := FPopupMenu
end;

function THmsMainFormControl.CreateControl(aParent: THandle; const aReserved: IUnknown; var aControl: THandle): HResult; safecall;
var
  ParentClientRect: TRect;
begin
  if RegistryKeyExists(HKEY_CLASSES_ROOT, 'CLSID\' + GuidToString(CLASS_VLCPlugin2)) then begin
    if FControl = nil then begin
      FControl := TWinControl.CreateParented(aParent);
      GetClientRect(aParent, ParentClientRect);
      FControl.BoundsRect := ParentClientRect;
      FPlayer := TVLCPlugin2.Create(FControl);
      FPlayer.Parent := FControl;
      FPlayer.Align := alClient;
      CreatePopupMenu
    end;
    aControl := FControl.Handle;
    Result := S_OK
  end else begin
    aControl := 0;
    Result := S_FALSE
  end
end;

function THmsMainFormControl.DestroyControl(aControl: THandle): HResult; safecall;
begin
  FreeAndNil(FControl);
  Result := S_OK
end;

function THmsMainFormControl.GetCaption(var aCaption: OleVariant): HResult; safecall;
begin
  aCaption := csAddonTitle;
  Result := S_OK
end;

function THmsMainFormControl.SetFocus: HResult;
begin
  if FPlayer <> nil then
    Windows.SetFocus(FPlayer.Handle);
  Result := S_OK
end;

function THmsMainFormControl.ExecuteAction(const aActionName: OleVariant; const aActionParameters: OleVariant): HResult; safecall;
var
  sActionName, sFileName: string;
begin
  Result := E_FAIL;
  try
    if FPlayer <> nil then begin
      sActionName := VarToStr(aActionName);
      if SameText(sActionName, 'open') then begin
        if VarIsStr(aActionParameters) then begin
          sFileName := VarToStr(aActionParameters);
          if Pos('://', sFileName) = 0 then
            sFileName := 'file:///' + StringReplace(sFileName, '\', '/', [rfReplaceAll]);
          FPlayer.Playlist.Stop;
          FPlayer.Playlist.Clear;
          FPlayer.Playlist.Add(sFileName, '', Null);
          FPlayer.Playlist.Play;
          Result := S_OK
        end
      end
    end
  except
    on E: Exception do
      Application.MessageBox(PChar(E.Message), '', MB_ICONERROR + MB_OK)
  end
end;

function THmsMainFormControl.GetActionCount(var aCount: Integer): HResult; safecall;
begin
  Result := E_NOTIMPL
end;

function THmsMainFormControl.GetActionInfo(aIndex: Integer; var aActionName, aActionTitle, aActionDescription: OleVariant): HResult; safecall;
begin
  Result := E_NOTIMPL
end;

function HmsGetAddonList(out aAddonList: IHmsAddonList): HResult; stdcall;
begin
  aAddonList := THmsAddonList.Create;
  Result := S_OK
end;

exports
  HmsGetAddonList;

end.
